from habutax.forms import ty2021
from habutax.forms import ty2022
from habutax.forms import ty2023

available_forms = {
    2021: ty2021.available_forms,
    2022: ty2022.available_forms,
    2023: ty2023.available_forms,
}
